@@ -1,7 +1,7 @@
 ##
 ## This file is part of the libsigrokdecode project.
 ##
-## Copyright (C) 2012-2014 Uwe Hermann <uwe@hermann-uwe.de>
+## Copyright (C) 2012-2016 Uwe Hermann <uwe@hermann-uwe.de>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -14,8 +14,7 @@
 ## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
-## along with this program; if not, write to the Free Software
-## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+## along with this program; if not, see <http://www.gnu.org/licenses/>.
 ##
 
 import sigrokdecode as srd
@@ -26,7 +25,7 @@ class SamplerateError(Exception):
     pass
 
 class Decoder(srd.Decoder):
-    api_version = 2
+    api_version = 3
     id = 'dcf77'
     name = 'DCF77'
     longname = 'DCF77 time protocol'
@@ -68,9 +67,6 @@ class Decoder(srd.Decoder):
     def __init__(self):
         self.samplerate = None
         self.state = 'WAIT FOR RISING EDGE'
-        self.oldpins = None
-        self.oldval = None
-        self.samplenum = 0
         self.ss_bit = self.ss_bit_old = self.es_bit = self.ss_block = 0
         self.datebits = []
         self.bitcount = 0 # Counter for the DCF77 bits (0..58)
@@ -244,21 +240,13 @@ class Decoder(srd.Decoder):
         else:
             raise Exception('Invalid DCF77 bit: %d' % c)
 
-    def decode(self, ss, es, data):
+    def decode(self):
         if not self.samplerate:
             raise SamplerateError('Cannot decode without samplerate.')
-        for (self.samplenum, pins) in data:
-
-            # Ignore identical samples early on (for performance reasons).
-            if self.oldpins == pins:
-                continue
-            self.oldpins, (val,) = pins, pins
-
+        while True:
             if self.state == 'WAIT FOR RISING EDGE':
                 # Wait until the next rising edge occurs.
-                if not (self.oldval == 0 and val == 1):
-                    self.oldval = val
-                    continue
+                self.wait({0: 'r'})
 
                 # Save the sample number where the DCF77 bit begins.
                 self.ss_bit = self.samplenum
@@ -283,9 +271,7 @@ class Decoder(srd.Decoder):
 
             elif self.state == 'GET BIT':
                 # Wait until the next falling edge occurs.
-                if not (self.oldval == 1 and val == 0):
-                    self.oldval = val
-                    continue
+                self.wait({0: 'f'})
 
                 # Save the sample number where the DCF77 bit ends.
                 self.es_bit = self.samplenum
@@ -309,5 +295,3 @@ class Decoder(srd.Decoder):
                     self.bitcount += 1
 
                 self.state = 'WAIT FOR RISING EDGE'
-
-            self.oldval = val
